/******************************************************************************
* To the extent possible under law, Sebastian Roith
* <https://www.roith-unterrichtet.de> has waived all copyright and related or
* neighboring rights to this script. This work is published from Germany.
******************************************************************************/

/**
* Klasse für einfache Programmieraufgaben mit mehreren Teilaufgaben
*/
function LmsConnector(
    node        // DOM-Knoten, der für die Benutzeroberfläche genutzt werden soll
) {
    this.upgradeLicenceBtn();
    this._ui = new TaskUI(node);
    this.connect();
    this._ui.onchange = (() => {
        if (this._lastSavedFailedCount != this._ui.fineCount || this._lastSavedDone != this._ui.done) {
            this.saveData();
            return;
        }
        if (this._saveTimeout !== undefined)
            clearTimeout(this._saveTimeout);
        this._saveTimeout = setTimeout(() => this.saveData(), 3000);
    });
}

// Genutzte TaskUI-Instanz
LmsConnector.prototype._ui = null;

// Zuletzt gespeicherter Wert für Fehlerpunkteanzahl
LmsConnector.prototype._lastSavedFailedCount = 0;

// Zuletzt gespeicherter Wert für vollständige Aufgabenbearbeitung
LmsConnector.prototype._lastSavedDone = false;

// Timeout für zeitversetztes Speichern
LmsConnector.prototype._saveTimeout = undefined;

/**
* Schaltfläche zum Einblenden der Lizenz-Infos anzeigen
* @return undefined
*/
LmsConnector.prototype.upgradeLicenceBtn = function() {
    new ScoTools.InfoPageLink();
}

/**
* Versuchen, eine Verbindung zur Lernplattform aufzubauen
* @return undefined
*/
LmsConnector.prototype.connect = function() {
    if (typeof ScoTools.LMSLink == 'function') {
        var self = this;
        this.lmslink = new ScoTools.LMSLink();
        this.lmslink.onnew = (function() {
            self.lmsValuesInit();
        });
        this.lmslink.start();
        // Werte laden
        try {
            const suspendData = this.lmslink.suspendData;
            const indexComma = suspendData.indexOf(',');
            const failedRaw = +((indexComma > -1) ? suspendData.substring(0, indexComma) : suspendData);
            const failedCount = Number.isInteger(failedRaw) ? failedRaw : 0;
            const code = (indexComma > -1) ? suspendData.substring(1 + indexComma) : '';
            this._lastSavedDone = this.getDone();
            this._lastSavedFailedCount = failedCount;
            this._ui.loadValues(this._lastSavedDone, failedCount, code);
        } catch(e) {
            this.lmsValuesInit();
        }
    }
    if (!this.lmslink.ready)
        this._ui.addGeneralMsg('Verbindungsaufnahme zur Lernplattform fehlgeschlagen - Fortschritte können nicht gespeichert werden', 'msg_warn');
}

/**
* Beim ersten Start des Lernpaketes Anfangswerte festlegen
* @return undefined
*/
LmsConnector.prototype.lmsValuesInit = function() {
    this.saveData();
}

// Genutzte Instanz von LMSLink
LmsConnector.prototype.lmslink = { 'ready': false, lessonLocation: '0,' };

/**
* Ermittelt, ob die Aufgabe bereits erfolgreich gelöst wurde
* @return Boolean       Aufgabe bereits gelöst?
*/
LmsConnector.prototype.getDone = function() {
    return (this.lmslink.ready && !this.lmslink.isIncomplete());
}

/**
* Prüft, ob Veränderungen am Lernpaket gespeichert werden sollen
* @return Boolean       Kann und soll gespeichert werden?
*/
LmsConnector.prototype.canSave = function() {
    return (!this.lmslink.ready || this.lmslink.isNormalIncomplete());
}

/**
* Bearbeitungszeit speichern
* @return undefined
*/
LmsConnector.prototype.reportTime = function() {
    this.lmslink.ready && this.lmslink.reportTime();
}

/**
* Punktestand und Status speichern
* @return undefined
*/
LmsConnector.prototype.saveData = function() {
    if (this._saveTimeout !== undefined) {
        clearTimeout(this._saveTimeout);
        this._saveTimeout = undefined;
    }
    if (!this.canSave()) {
        this.reportTime();
        return;
    }
    const data = {
        lessonStatus: this._ui.done ? 'completed' : 'incomplete'
    };
    if (this._ui.done) {
        data.scoreMin = 0;
        data.scoreMax = 100;
        data.score = Math.max(0, 100 - this._ui.fineCount);
    }
    this._lastSavedDone = this._ui.done;
    this._lastSavedFailedCount = this._ui.fineCount;
    data.suspendData = this._ui.fineCount + ',' + this._ui.code;
    this.save(data);
}

/**
* Einen Wert vom LMS lesen
* @return undefined
*/
LmsConnector.prototype.read = function(
    key     // Attributname von LMSLink-Instanz, die gelesen werden soll
) {
    return (this.lmslink.ready ? this.lmslink[key] : undefined);
}

/**
* Einen Wert vom LMS speichern lassen
* @return undefined
*/
LmsConnector.prototype.write = function(
    key,        // Attributname von LMSLink-Instanz, unter dem der Wert gespeichert werden soll
    value       // Zu speichernder Wert
) {
    if (this.lmslink.ready)
        this.lmslink[key] = value;
}

/**
* Zugleich mehrere Werte ändern, die Zeit speichern und dann erst Commit an das LMS senden
* @return undefined
*/
LmsConnector.prototype.save = function(
    data        // Objekt mit zu speichernden Daten
) {
    if (this.lmslink.ready)
        this.lmslink.save(data);
}
